import os
#from django.template import Context, loader
#from django.http import HttpResponseRedirect
from django.shortcuts import (
    get_object_or_404, render_to_response)
from django.utils.safestring import SafeUnicode
#from django.core.urlresolvers import reverse
from thuvien.collection.models import Collection, Subject, Volume
from thuvien.collection.tables import VolumeTable
from PIL import Image
from textwrap import dedent


class DiskNotMounted(Exception):
    def __init__(self, value):
        self.value = value
    def __unicode__(self):
        return "The disk is not mounted: %s" % self.value


def mark_safe(md):
    for key in md.__dict__.keys():
        if key.endswith('hn') and md.__dict__[key]:
            md.__dict__[key] = SafeUnicode(md.__dict__[key])


def _get_images(volume):
    mount = volume.storage.mount_point
    image_dir = os.path.join(mount, volume.nlvnpf, 'jpeg')
    image_path = os.path.join('site_media', os.path.split(mount)[1],
                              volume.nlvnpf, 'jpeg')
    if not os.path.exists(image_dir):
        raise DiskNotMounted(mount)
    images = sorted([i for i in os.listdir(image_dir) if i.endswith('.jpg')])
    epub_name = "%s.epub" % volume.nlvnpf
    epub = None
    if os.path.exists(
        os.path.join(mount, volume.nlvnpf, epub_name)):
        epub = os.path.join('site_media', os.path.split(mount)[1],
                            volume.nlvnpf, epub_name)
    return images, image_path, image_dir, epub


def volume_index(request, object_id):
    template = 'collection/volume-index.html'
    error_message = ''
    volume = get_object_or_404(Volume, pk=object_id)
    try:
        images, image_path, image_dir, epub = _get_images(volume)
        cover=images[-2]

    except DiskNotMounted, e:
        error_message = e
        print error_message
        images = []
        image_path = 'Unmounted'
        cover=None
    mark_safe(volume.md)
    return render_to_response(
        template,
        dict(object=volume,
             error_message=error_message,
             path=image_path,
             cover=cover,
             epub=epub,
             num_images=len(images)))


def page_thumbnails_frame(request, object_id):
    template = 'collection/pages-frame.html'
    return pages(request, object_id, template)


def pages(request, object_id, template=None):
    if template is None:
        template = 'collection/volume-pages.html'
    volume = get_object_or_404(Volume, pk=object_id)
    images, image_path, image_dir, epub = _get_images(volume)
    return render_to_response(
        template,
        dict(object=volume,
             path=image_path,
             images=images))


def create_maps(width, height, progress_bar_height, percent, page_num,
                num_images, offset=0):
    midpoint = width / 2
    areas = []
    area_tmpl = dedent("""\
  <area shape="rect"
        coords="%(left)d, %(top)d, %(right)d, %(bottom)d"
        href="%(href)d"/>
""")
    map_tmpl = dedent("""\
<map name="%(mapname)s">
  %(areas)s
</map>
""")

    # Left page map to previous page.
    if page_num > 0:
        areas.append(area_tmpl % dict(
            left=0, top=0, right=midpoint, bottom=height-offset,
            href=page_num-1))
    # Right page map to next page.
    if page_num < num_images:
        areas.append(area_tmpl % dict(
            left=midpoint, top=0, right=width, bottom=height-offset,
            href=page_num+1))

    page_map = map_tmpl % dict(
        mapname="pagemap",
        areas="\n".join(areas))

    # Now create maps across the bottom, evenly divided by the number of
    # pages.
    def make_progress_map(start, end, width, heigth):
        areas = []
        num_images = max(end - start + 1, 1)
        delta = width / num_images
        left = 0
        bottom = height
        top = 0
        for i in xrange(num_images):
            pagenum = start + i
            right = left + delta
            areas.append(area_tmpl % dict(
                left=left, top=top, right=right, bottom=bottom,
                href=pagenum))
            left = right
        return areas
    done_width = width*(percent/100.0)
    areas = make_progress_map(
        1, page_num-1, done_width, progress_bar_height)
    progress_map_done = map_tmpl % dict(
        mapname="progress_map_done", areas="\n".join(areas))

    areas = make_progress_map(
        page_num, num_images, width-done_width, progress_bar_height)
    progress_map_undone = map_tmpl % dict(
        mapname="progress_map_undone", areas="\n".join(areas))

    return (page_map, progress_map_done, progress_map_undone)


def page(request, volume_id, page_id):
    template = 'collection/page-index.html'
    volume = get_object_or_404(Volume, pk=volume_id)
    images, image_path, image_dir, epub = _get_images(volume)
    num_images = len(images) - 3
    page_num = int(page_id)
    mark_safe(volume.md)
    percent = int(float(page_num) / num_images * 100)
    image=images[page_num - 1]
    small_image = os.path.join(image_path, image)
    full_image = small_image.replace('jpeg', 'large')
    i = Image.open(os.path.join(image_dir, image))
    width, height = i.size
    progress_bar_height = 15
    page_map, done, undone = create_maps(
        width, height, progress_bar_height, percent, page_num, num_images)
    return render_to_response(
        template,
        dict(object=volume,
             done=percent,
             undone=100-percent,
             page_id=page_id,
             last_id=num_images,
             path=image_path,
             small_image=small_image,
             full_image=full_image,
             pg_bar_height=progress_bar_height,
             page_map=page_map,
             progress_map_done=done,
             progress_map_undone=undone))


def chunkify(l, start=0, step=20):
    chunks = []
    left = start
    for i in range(start, len(l), step):
        chunks.append(l[left:left+step])
        left += step
    return chunks


def collection(request, object_id):
    """Show a collection, listing all of the volumes in it."""
    template = 'collection/collection-index.html'
    collection = get_object_or_404(Collection, pk=object_id)
    volumes = list(collection.volume_set.all())
    volumes = sorted(volumes, key=lambda x: x.sort_key())
    return render_to_response(
        template,
        dict(collection=collection,
             chunks=chunkify(volumes)))


def subjects(request, object_id):
    """Show a collection, listing all of the volumes in it."""
    template = 'collection/subjects.html'
    collection = get_object_or_404(Collection, pk=object_id)
    subjects = Subject.objects.all()
    for subject in subjects:
        subject.count = (
            collection.volume_set.filter(md__subject=subject).count())
    return render_to_response(
        template,
        dict(collection=collection,
             subjects=subjects))


def subject(request, object_id, subject_id):
    """Show a collection and subject, listing all of the volumes in it."""

    template = 'collection/subject.html'
    collection = get_object_or_404(Collection, pk=object_id)
    subject = get_object_or_404(Subject, pk=subject_id)
    volumes = collection.volume_set.filter(md__subject=subject)
    volumes_table = VolumeTable(volumes, order_by=request.GET.get('sort'))
    return render_to_response(
        template,
        dict(collection=collection,
             subject=subject,
             table=volumes_table))
